"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const FetchContext_1 = __importDefault(require("./FetchContext"));
const types_1 = require("./types");
const utils_1 = require("./utils");
// TODO: handle context.graphql
function useFetch(urlOrOptions, optionsNoURLs) {
    const context = react_1.useContext(FetchContext_1.default);
    utils_1.invariant(!!urlOrOptions || !!context.url, 'The first argument of useFetch is required unless you have a global url setup like: <Provider url="https://example.com"></Provider>');
    let url = context.url || '';
    let options = {};
    let onMount = false;
    // let timeout: number = 10 // TODO: not implemented
    const handleUseFetchOptions = react_1.useCallback((useFetchOptions) => {
        const opts = useFetchOptions || {};
        if ('onMount' in opts)
            onMount = opts.onMount;
        // if (opts.timeout) timeout = opts.timeout
        if ('url' in context)
            url = context.url;
        if ('url' in opts)
            url = opts.url;
    }, []);
    // ex: useFetch('https://url.com', { onMount: true })
    if (utils_1.isString(urlOrOptions) && utils_1.isObject(optionsNoURLs)) {
        url = urlOrOptions;
        options = utils_1.pullOutRequestInit(optionsNoURLs);
        handleUseFetchOptions(optionsNoURLs);
        // ex: useFetch('https://url.com')
    }
    else if (utils_1.isString(urlOrOptions) && optionsNoURLs === undefined) {
        url = urlOrOptions;
        // ex: useFetch({ onMount: true }) OR useFetch({ url: 'https://url.com' })
    }
    else if (utils_1.isObject(urlOrOptions)) {
        utils_1.invariant(!optionsNoURLs, 'You cannot have a 2nd parameter of useFetch when your first argument is a object config.');
        let optsWithURL = urlOrOptions;
        utils_1.invariant(!!context.url || !!optsWithURL.url, 'You have to either set a URL in your options config or set a global URL in your <Provider url="https://url.com"></Provider>');
        options = utils_1.pullOutRequestInit(urlOrOptions);
        handleUseFetchOptions(urlOrOptions);
    }
    // Provider ex: useFetch({ url: 'https://url.com' }) -- (overwrites global url)
    // TODO - Provider: arg1 = oldGlobalOptions => ({ my: 'new local options'}) (overwrite all global options for this instance of useFetch)
    const [data, setData] = react_1.useState();
    const [loading, setLoading] = react_1.useState(onMount);
    const [error, setError] = react_1.useState();
    const controller = react_1.useRef();
    const makeFetch = react_1.useCallback((method) => {
        if ('AbortController' in window) {
            controller.current = new AbortController();
            options.signal = controller.current.signal;
        }
        let route = '';
        function doFetch(routeOrBody, body) {
            return __awaiter(this, void 0, void 0, function* () {
                // TODO: do the ts types handle if routeOrBody AND body are both objects it will error?
                // if not, an invariant(!isObject(routeOrBody) && !isObject(body), 'both arguments cannot be an object')
                // POST, PATCH, PUT, DELETE
                if (method !== types_1.HTTPMethod.GET) { // TODO: add OPTIONS later
                    // ex: request.post('/no', { freaking: 'way' })
                    // ex: reqeust.post('/yes-way')
                    if (utils_1.isString(routeOrBody)) {
                        route = routeOrBody;
                        options.body = JSON.stringify(body || {});
                        // ex: request.post({ no: 'way' })
                    }
                    else if (utils_1.isObject(routeOrBody)) {
                        utils_1.invariant(!body, `If first argument of ${method.toLowerCase()}() is an object, you cannot have a 2nd argument. 😜`);
                        options.body = JSON.stringify(routeOrBody || {});
                    }
                    // GET
                }
                else {
                    utils_1.invariant(!utils_1.isObject(routeOrBody), 'Cannot pass a request body in a GET request');
                    // ex: request.get('/no?freaking=way')
                    if (utils_1.isString(routeOrBody))
                        route = routeOrBody;
                }
                try {
                    setLoading(true);
                    const response = yield fetch(url + route, Object.assign({ method }, context.options, options, { headers: Object.assign({ 
                            // default content types http://bit.ly/2N2ovOZ
                            // Accept: 'application/json', 
                            'Content-Type': 'application/json' }, (context.options || {}).headers, options.headers) }));
                    let data = null;
                    try {
                        data = yield response.json();
                    }
                    catch (err) {
                        data = yield response.text();
                    }
                    setData(data);
                }
                catch (err) {
                    if (err.name !== 'AbortError')
                        setError(err);
                }
                finally {
                    controller.current = null;
                    setLoading(false);
                }
            });
        }
        return doFetch;
    }, [url]);
    const get = react_1.useCallback(makeFetch(types_1.HTTPMethod.GET), []);
    const post = react_1.useCallback(makeFetch(types_1.HTTPMethod.POST), []);
    const patch = react_1.useCallback(makeFetch(types_1.HTTPMethod.PATCH), []);
    const put = react_1.useCallback(makeFetch(types_1.HTTPMethod.PUT), []);
    const del = react_1.useCallback(makeFetch(types_1.HTTPMethod.DELETE), []);
    const query = react_1.useCallback((query, variables) => post({ query, variables }), []);
    const mutate = react_1.useCallback((mutation, variables) => post({ mutation, variables }), []);
    const abort = react_1.useCallback(() => {
        controller.current && controller.current.abort();
    }, []);
    const request = react_1.useMemo(() => ({ get, post, patch, put, del, delete: del, abort, query, mutate }), []);
    // handling onMount
    react_1.useEffect(() => {
        const methodName = ((options.method || '') || types_1.HTTPMethod.GET).toUpperCase();
        if (!!url && onMount && methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodName.toLowerCase()];
            req(url, options.body);
        }
        else if (!url && onMount && methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodName.toLowerCase()];
            req(options.body);
        }
        else if (!!url && onMount) {
            const req = request[methodName.toLowerCase()];
            req(url);
        }
        else if (onMount) {
            const req = request[methodName.toLowerCase()];
            req();
        }
    }, []);
    return Object.assign([data, loading, error, request], Object.assign({ data, loading, error, request }, request));
}
exports.useFetch = useFetch;
exports.default = useFetch;
//# sourceMappingURL=useFetch.js.map